#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2016 The Aerospace Corporation.
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

from gnuradio import eng_notation
from gnuradio import gr
from gnuradio import zeromq
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from optparse import OptionParser
import osmosdr
import time


class rtl_to_zmq_c(gr.top_block):

    def __init__(self, addr='tcp://127.0.0.1:12345', carrier_fc=0, device_string='rtl', gain=0, samp_rate=32e3):
        gr.top_block.__init__(self, "Rtl To Zmq C")

        ##################################################
        # Parameters
        ##################################################
        self.addr = addr
        self.carrier_fc = carrier_fc
        self.device_string = device_string
        self.gain = gain
        self.samp_rate = samp_rate

        ##################################################
        # Blocks
        ##################################################
        self.zeromq_push_sink_0 = zeromq.push_sink(gr.sizeof_gr_complex, 1, addr, 100, False, -1)
        self.rtlsdr_source_0 = osmosdr.source( args="numchan=" + str(1) + " " + '' )
        self.rtlsdr_source_0.set_sample_rate(samp_rate)
        self.rtlsdr_source_0.set_center_freq(carrier_fc, 0)
        self.rtlsdr_source_0.set_freq_corr(0, 0)
        self.rtlsdr_source_0.set_dc_offset_mode(0, 0)
        self.rtlsdr_source_0.set_iq_balance_mode(0, 0)
        self.rtlsdr_source_0.set_gain_mode(False, 0)
        self.rtlsdr_source_0.set_gain(gain, 0)
        self.rtlsdr_source_0.set_if_gain(0, 0)
        self.rtlsdr_source_0.set_bb_gain(0, 0)
        self.rtlsdr_source_0.set_antenna('', 0)
        self.rtlsdr_source_0.set_bandwidth(0, 0)
          

        ##################################################
        # Connections
        ##################################################
        self.connect((self.rtlsdr_source_0, 0), (self.zeromq_push_sink_0, 0))    

    def get_addr(self):
        return self.addr

    def set_addr(self, addr):
        self.addr = addr

    def get_carrier_fc(self):
        return self.carrier_fc

    def set_carrier_fc(self, carrier_fc):
        self.carrier_fc = carrier_fc
        self.rtlsdr_source_0.set_center_freq(self.carrier_fc, 0)

    def get_device_string(self):
        return self.device_string

    def set_device_string(self, device_string):
        self.device_string = device_string

    def get_gain(self):
        return self.gain

    def set_gain(self, gain):
        self.gain = gain
        self.rtlsdr_source_0.set_gain(self.gain, 0)

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.rtlsdr_source_0.set_sample_rate(self.samp_rate)


def argument_parser():
    parser = OptionParser(usage="%prog: [options]", option_class=eng_option)
    parser.add_option(
        "", "--addr", dest="addr", type="string", default='tcp://127.0.0.1:12345',
        help="Set ZMQ address [default=%default]")
    parser.add_option(
        "", "--carrier-fc", dest="carrier_fc", type="eng_float", default=eng_notation.num_to_str(0),
        help="Set Carrier frequency (Hz) [default=%default]")
    parser.add_option(
        "", "--device-string", dest="device_string", type="string", default='uhd',
        help="Set Device string [default=%default]")
    parser.add_option(
        "", "--gain", dest="gain", type="eng_float", default=eng_notation.num_to_str(0),
        help="Set Gain (dB) [default=%default]")
    parser.add_option(
        "", "--samp-rate", dest="samp_rate", type="eng_float", default=eng_notation.num_to_str(32e3),
        help="Set Sample rate [default=%default]")
    return parser


def main(top_block_cls=rtl_to_zmq_c, options=None):
    if options is None:
        options, _ = argument_parser().parse_args()

    tb = top_block_cls(addr=options.addr, carrier_fc=options.carrier_fc, device_string=options.device_string, gain=options.gain, samp_rate=options.samp_rate)
    tb.start()
    tb.wait()


if __name__ == '__main__':
    main()
