import abc
from Queue import Queue

##
# @brief GOES Layer abstract class.
# @details Each layer of the LRIT packet processing is subclassed from this basic skeleton.
# @author Miguel Dominguez
# @date 2016
class GOESLayer(object):
    #Is an abstract base class
    __metaclass__ = abc.ABCMeta

    QUEUE_MAX_SIZE = 10 #Arbitrary, could be anything

    ##
    # @fn __init__
    # @brief Constructor.
    # @details Creates an input buffer.
    def __init__(self):
        self.inBuffer = Queue(self.QUEUE_MAX_SIZE)

    ##
    # @fn connect
    # @brief Connects an external buffer as output.
    # @param[in] q The output buffer, a Queue.
    def connect(self,q):
        self.outBuffer = q

    ##
    # @fn threadFunction
    # @brief Coordinates the input and output buffers.
    # @detail This is a default implementation for taking data from the input buffer,
    # processing it, and then passing output products to the output buffer, if they exist.
    # This function may be overridden if a given Layer needs to handle the buffers differently.
    # While called threadFunction, it can be passed to a thread, process, or even run inline.
    def threadFunction(self):
        while True:
            (inData,inOpsMsg) = self.inBuffer.get()
            if inData is not None:
                (outData,outOpsMsg) = self.processData(inData,inOpsMsg)
                #self.inBuffer.task_done()
                if outData:
                    self.outBuffer.put((outData,outOpsMsg))
            else:
                #self.inBuffer.task_done()
                self.outBuffer.put((None,None))
                break

    ##
    # @fn getInBuffer
    # @brief Getter for input buffer.
    # @param[out] The input buffer.
    def getInBuffer(self):
        return self.inBuffer

    ##
    # @fn processData
    # @brief Processes input data and operations messages from input buffer.
    # @detail This is an abstract method that must be overridden for every class
    # @param[in] data Expected data object for given layer.
    # @param[in] opsMsgs Expected operations messages for given layer.  These can be
    # warnings or notifications from previous layers that the current layer may need
    # to act upon.
    # @param[out] outData Should output processed output objects.
    # @param[out] outOpsMsg Should output processed operational messages, such as warnings
    # that future layers need to act upon.
    @abc.abstractmethod
    def processData(self,data,opsMsgs):
        raise NotImplementedError('must define processData function!')
