import sys
import os
import logging
import multiprocessing
from .GOESLayer import GOESLayer
from .files.LritFileBuilder import LritFileBuilder
from .headers.PrimaryHeader import PrimaryHeader

##
# @brief PPDU Layer class.
# @details Fifth layer of LRIT packet processing. Extracts Files from Transport
# frames.  Again, there is a many:many relationship between Transport frames and
# files.
# @author Miguel Dominguez
# @date 2016
class PPDULayer(GOESLayer):

    PREPENDED_JUNK_BYTES_COUNT = 10

    ##
    # @fn __init__
    # @brief Constructor.
    # @details Initializes the logger and file table
    def __init__(self):
        GOESLayer.__init__(self)
        self.logger = logging.getLogger(__name__)
        self.fileTable = dict()

    ##
    # @fn __isNewLritFile
    # @brief Checks whether the current transport frame is the
    # start of a new file
    # @details There are two cases:  A sequence flag of 1
    # indicates that this frame is the start of a new multi-frame
    # file, while a sequence flag of 3 indicates the start of a
    # complete, self-contained file within one frame
    # param[in] data the new Transport frame
    # param[out] Success Whether the Transport frame indicates a new
    # file
    def __isNewLritFile(self, data):
        return (data.sequenceFlag == 1) or (data.sequenceFlag == 3)
    ##
    # @fn __isCompleteLritFile
    # @brief Checks whether the current transport frame is the
    # start of a new file
    # @details There are two cases:  A sequence flag of w
    # indicates that this frame is the last of a multi-frame
    # file, while a sequence flag of 3 indicates the start of a
    # complete, self-contained file within one frame
    # param[in] data the new Transport frame
    # param[out] Success Whether the Transport frame indicates a new
    # file
    def __isCompleteLritFile(self, data):
        return (data.sequenceFlag == 2) or (data.sequenceFlag == 3)

    ##
    # @fn threadFunction
    # @brief Coordinates the input and output buffers.
    # @detail This is a default implementation for taking data from the input buffer,
    # processing it, and then passing output products to the output buffer, if they exist.
    # This function may be overridden if a given Layer needs to handle the buffers differently.
    # While called threadFunction, it can be passed to a thread, process, or even run inline.
    def threadFunction(self):
        while True:
            (inData, inOpsMsg) = self.inBuffer.get()
            if inData is not None:
                (outData, outOpsMsg) = self.processData(inData, inOpsMsg)
                #self.inBuffer.task_done()
                if outData:
                    if outData.headerDict[PrimaryHeader.headerIndex()].fileTypeCode == PrimaryHeader.EMWIN_FILE_TYPE_CODE:
                        self.outBuffer[1].put((outData, outOpsMsg))
                    else:
                        self.outBuffer[0].put((outData, outOpsMsg))
            else:
                #self.inBuffer.task_done()
                for slot in self.outBuffer:
                    self.outBuffer[slot].put((None, None))
                break

    ##
    # @fn processData
    # @brief Extracts files from Transport frames and performs some basic checks.
    # @detail Checks if there are CRC errors and flush the file if there are, otherwise
    # build files as Transport frames come in.
    # @param[in] data CPPDUMsg Object
    # @param[in] opsMsgs Information about CRC errors
    # @param[out] outData LritFile object
    # @param[out] outOpsMsg Filler, is ignored
    # @warning There are uncaught exceptions raised in processBuffer call here
    def processData(self, data, opsMsg):
        #self.logger.debug(data)
        if not opsMsg.getHasCrcErrors():
            if self.__isNewLritFile(data):
                self.fileTable[data.apid] = LritFileBuilder()
                #Inexplicable 10 bytes prepended to new file, not in the spec
                data.data = data.data[self.PREPENDED_JUNK_BYTES_COUNT:len(data.data)]
            if data.apid in self.fileTable:
                self.fileTable[data.apid].processBuffer(data.data)

            if self.__isCompleteLritFile(data) and data.apid in self.fileTable:
                output = self.fileTable[data.apid].file
                self.fileTable.pop(data.apid, None)
                return(output,True)
            else:
                return ([],True)
        else:
            self.logger.warn("Transport Frame has CRC Errors")
            return ([],False)
