#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2016 The Aerospace Corporation.
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

"""Location Data Intializer

This module initializes the data for locations to be used in filters_tab.py.
It uses 3 .dat files, located in the location_data directory:
    States.dat -- states and their abbreviations
    County_F.dat -- counties of land locations and their county codes
    County_Z.dat -- counties of sea locations and their county codes

Lists and dictionaries are created by these files for convenient access.
The lists and dictionaries created are shown below:
"""

import os.path as osp
_thisdir = osp.dirname(osp.abspath(__file__))
_default_location_dat_dir = _location_dat_dir = osp.join(_thisdir, 'location_data')

# default file names included
COUNTY_F = 'County_F.csv'
COUNTY_Z = 'County_Z.csv'
WFO_SITES= 'WfoSites.csv'
PRODUCTS = 'Products.csv'
STATES = 'States.csv'

import fileinput
from collections import namedtuple

# For filtering NEED:
# Product name -> code
# State name -> abbrev
# WFO site -> code
# TODO: these lists could be incorporated into a preview for products which 
# expand the filename into the full state, county, site and possibly map 
# thumbnail?

# public mappings
COUNTY_MAP = {}
STATE_MAP = {}
PRODUCT_MAP = {}
WFOSITE_MAP = {}

# read state info, map to name
StateRecord = namedtuple('StateRecord', 'abbrev, name')
with open(osp.join(_location_dat_dir, STATES), 'r') as _f:
    for _line in _f:
        _record = StateRecord(*_line.strip().split(','))
        STATE_MAP[_record.name] = _record

# read county/zone info, map to name
CountyRecord = namedtuple('CountyRecord', 'state, type, num, name, lon, lat')
for _line in fileinput.input([osp.join(_location_dat_dir, x) for x in (COUNTY_F, COUNTY_Z)]):
    _cols = _line.strip().split(',')
    # link state to record
    _staterecord = None
    try:
        _staterecord = [v for v in STATE_MAP.itervalues() if v.abbrev == _cols[0][:2]][0]
    except IndexError:  # state abbreviation not found
        pass
    # add state entry to avoid ambiguity and mark each state's list
    if _cols[1] == 'Any County': _cols[1] = '** %s Counties **' % _staterecord.abbrev
    # skip Any County entries since no selection is "Any County"
    #if _cols[1] == 'Any County': continue
    _record = CountyRecord(_staterecord, _cols[0][2], _cols[0][-3:],
                           _cols[1], float(_cols[2]), float(_cols[3]))
    COUNTY_MAP[_record.name] = _record
        
# read products
ProductRecord = namedtuple('ProductRecord', 'abbrev, code, f3, f4, f5, name')
with open(osp.join(_location_dat_dir, PRODUCTS), 'r') as _f:
    for _line in _f:
        _cols = _line.strip().split(',')
        _record = ProductRecord(_cols[0], _cols[1], 
                              int(_cols[2]), int(_cols[3]), int(_cols[4]),
                              _cols[5])
        PRODUCT_MAP[_record.name] = _record

# WFO records
WFORecord = namedtuple('WFORecord', 'abbrev, name, state')  # state is abbrev or XX
with open(osp.join(_location_dat_dir, WFO_SITES), 'r') as _f:
    for _line in _f:
        _record = WFORecord(*_line.strip().split(','))
        WFOSITE_MAP[_record.name] = _record

# convenience structures
SORTED_STATE_LIST = sorted(STATE_MAP.keys())
SORTED_PRODUCT_LIST = sorted(PRODUCT_MAP.keys())
SORTED_OFFICE_LIST = sorted(WFOSITE_MAP.keys())
# state mapped to list of counties
_state_name_to_county_set = {k: set() for k in SORTED_STATE_LIST}
for v in COUNTY_MAP.itervalues():
    _state_name_to_county_set[v.state.name].add(v.name)
STATE_NAME_TO_COUNTY_LIST = {k: sorted(list(v)) for k, v in _state_name_to_county_set.iteritems()}

# print maps
from pprint import pprint
if __name__ == "__main__":
    #pprint(COUNTY_MAP)
    #pprint(STATE_MAP)
    pprint(PRODUCT_MAP)
    #pprint(WFOSITE_MAP)
    #pprint(STATE_NAME_TO_COUNTY_LIST)
